<?php

namespace Drupal\crm_core_commerce\Mapper;


use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\crm_core_contact\IndividualInterface;

class CrmCoreIndividualMapper implements CrmCoreIndividualMapperInterface {

  const INDIVIDUAL_TYPE = "individual";

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new EntityRouteProviderSubscriber instance.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * @inheritdoc
   */
  public function mapOrderToIndividual(OrderInterface $order): IndividualInterface {
    /** @var \Drupal\crm_core_contact\Entity\IndividualType $type */
    $type = $this->entityTypeManager->getStorage('crm_core_individual_type')->load(static::INDIVIDUAL_TYPE);
    $primary_fileds = $type->getPrimaryFields();

    /** @var IndividualInterface $individual */
    $individual = $this->entityTypeManager->getStorage('crm_core_individual')
      ->loadByProperties([$primary_fileds['email'] => $order->getEmail()]);

    if (empty($individual) || $individual === NULL) {
      $individual = $this->createNewIndividual($order, $primary_fileds);
    }
    else {
      $individual = $this->updateIndividual($order, reset($individual), $primary_fileds);
    }

    return $individual;
  }

  protected function createNewIndividual(OrderInterface $order, array $primary_fileds): IndividualInterface {
    $profile = $order->getBillingProfile();

    /** @var IndividualInterface $individual */
    $individual = $this->entityTypeManager->getStorage('crm_core_individual')
      ->create(
        [
          'type' => static::INDIVIDUAL_TYPE,
          $primary_fileds['email'] => $order->getEmail(),
          $primary_fileds['phone'] => $profile->address->phone,
          $primary_fileds['address'] => $profile->address->getValue()[0],
          'name' => [
            'given' => $profile->address->given_name,
            'family' => $profile->address->family_name,
          ],
        ]);

    return $individual;
  }

  protected function updateIndividual(OrderInterface $order, IndividualInterface $individual, array $primary_fileds): IndividualInterface {
    $profile = $order->getBillingProfile();

    $individual->set($primary_fileds['address'], $profile->address->getValue()[0]);
    $individual->set($primary_fileds['email'], $order->getEmail());
    $individual->set($primary_fileds['phone'], $profile->address->phone);
    $individual->set('name',
      [
        'name' => $profile->address->given_name,
        'family' => $profile->address->family_name,
      ]);

    return $individual;
  }

}